"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// TODO: make this more generic and reuse code for static resources and this
const path_1 = require("path");
const glob_1 = require("glob");
const vscode_languageserver_1 = require("vscode-languageserver");
const util_1 = require("util");
const fs = __importStar(require("fs-extra"));
const glob = util_1.promisify(glob_1.Glob);
const CONTENT_ASSET_DECLARATION_FILE = '.sfdx/typings/lwc/contentassets.d.ts';
const CONTENT_ASSET_INDEX_FILE = '.sfdx/indexes/lwc/contentassets.json';
let CONTENT_ASSETS = new Set();
function resetContentAssets() {
    CONTENT_ASSETS.clear();
}
exports.resetContentAssets = resetContentAssets;
function getResourceName(resourceMetaFile) {
    const resourceFile = resourceMetaFile.substring(0, resourceMetaFile.lastIndexOf('-'));
    return path_1.parse(resourceFile).name;
}
async function updateContentAssetIndex(updatedFiles, { workspaceRoots }, writeConfigs = true) {
    let didChange = false;
    for (const f of updatedFiles) {
        if (f.uri.endsWith('.asset-meta.xml')) {
            if (f.type === vscode_languageserver_1.FileChangeType.Created) {
                didChange = true;
                CONTENT_ASSETS.add(getResourceName(f.uri));
            }
            else if (f.type === vscode_languageserver_1.FileChangeType.Deleted) {
                CONTENT_ASSETS.delete(getResourceName(f.uri));
                didChange = true;
            }
        }
    }
    if (didChange) {
        return processContentAssets(workspaceRoots[0], writeConfigs);
    }
}
exports.updateContentAssetIndex = updateContentAssetIndex;
function processContentAssets(workspace, writeConfig) {
    if (CONTENT_ASSETS.size > 0 && writeConfig) {
        return fs.writeFile(path_1.join(workspace, CONTENT_ASSET_DECLARATION_FILE), generateTypeDeclarations());
    }
}
async function indexContentAssets(context, writeConfigs) {
    const { workspaceRoots } = context;
    const workspace = workspaceRoots[0];
    const { sfdxPackageDirsPattern } = await context.getSfdxProjectConfig();
    const CONTENT_ASSET_GLOB_PATTERN = `${sfdxPackageDirsPattern}/**/contentassets/*.asset-meta.xml`;
    try {
        if (initContentAssetsIndex(workspace)) {
            return Promise.resolve();
        }
        const files = await glob(CONTENT_ASSET_GLOB_PATTERN, { cwd: workspaceRoots[0] });
        for (const file of files) {
            CONTENT_ASSETS.add(getResourceName(file));
        }
        return processContentAssets(workspaceRoots[0], writeConfigs);
    }
    catch (err) {
        console.log(`Error queuing up indexing of content resources. Error details:`, err);
        throw err;
    }
}
exports.indexContentAssets = indexContentAssets;
const generateTypeDeclarations = () => Array.from(CONTENT_ASSETS)
    .sort()
    .map(generateTypeDeclaration)
    .join('');
const generateTypeDeclaration = (resourceName) => `declare module "@salesforce/contentAssetUrl/${resourceName}" {
    var ${resourceName}: string;
    export default ${resourceName};
}
`;
function initContentAssetsIndex(workspace) {
    const indexPath = path_1.join(workspace, CONTENT_ASSET_INDEX_FILE);
    const shouldInit = fs.existsSync(indexPath);
    if (shouldInit) {
        const indexJsonString = fs.readFileSync(indexPath, 'utf8');
        const staticIndex = JSON.parse(indexJsonString);
        CONTENT_ASSETS = new Set(staticIndex);
        return true;
    }
    else {
        return false;
    }
}
function persistContentAssets(context) {
    const { workspaceRoots } = context;
    const indexPath = path_1.join(workspaceRoots[0], CONTENT_ASSET_INDEX_FILE);
    const index = Array.from(CONTENT_ASSETS);
    const indexJsonString = JSON.stringify(index);
    fs.writeFile(indexPath, indexJsonString);
}
exports.persistContentAssets = persistContentAssets;
//# sourceMappingURL=content-assets-util.js.map